@@ -52,7 +52,7 @@ static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify
 ValaCCodeWriter* vala_ccode_writer_construct (GType object_type, const char* filename) {
 	ValaCCodeWriter* self;
 	g_return_val_if_fail (filename != NULL, NULL);
-	self = ((ValaCCodeWriter*) (g_type_create_instance (object_type)));
+	self = (ValaCCodeWriter*) g_type_create_instance (object_type);
 	vala_ccode_writer_set_filename (self, filename);
 	return self;
 }
@@ -76,13 +76,13 @@ gboolean vala_ccode_writer_open (ValaCCodeWriter* self) {
 		char* _tmp0;
 		FILE* _tmp1;
 		_tmp0 = NULL;
-		self->priv->temp_filename = (_tmp0 = g_strdup_printf ("%s.valatmp", self->priv->_filename), (self->priv->temp_filename = (g_free (self->priv->temp_filename), NULL)), _tmp0);
+		self->priv->temp_filename = (_tmp0 = g_strdup_printf ("%s.valatmp", self->priv->_filename), self->priv->temp_filename = (g_free (self->priv->temp_filename), NULL), _tmp0);
 		_tmp1 = NULL;
-		self->priv->stream = (_tmp1 = fopen (self->priv->temp_filename, "w"), (self->priv->stream == NULL ? NULL : (self->priv->stream = (fclose (self->priv->stream), NULL))), _tmp1);
+		self->priv->stream = (_tmp1 = fopen (self->priv->temp_filename, "w"), (self->priv->stream == NULL) ? NULL : (self->priv->stream = (fclose (self->priv->stream), NULL)), _tmp1);
 	} else {
 		FILE* _tmp2;
 		_tmp2 = NULL;
-		self->priv->stream = (_tmp2 = fopen (self->priv->_filename, "w"), (self->priv->stream == NULL ? NULL : (self->priv->stream = (fclose (self->priv->stream), NULL))), _tmp2);
+		self->priv->stream = (_tmp2 = fopen (self->priv->_filename, "w"), (self->priv->stream == NULL) ? NULL : (self->priv->stream = (fclose (self->priv->stream), NULL)), _tmp2);
 	}
 	return (self->priv->stream != NULL);
 }
@@ -97,7 +97,7 @@ void vala_ccode_writer_close (ValaCCodeWriter* self) {
 	g_return_if_fail (self != NULL);
 	inner_error = NULL;
 	_tmp0 = NULL;
-	self->priv->stream = (_tmp0 = NULL, (self->priv->stream == NULL ? NULL : (self->priv->stream = (fclose (self->priv->stream), NULL))), _tmp0);
+	self->priv->stream = (_tmp0 = NULL, (self->priv->stream == NULL) ? NULL : (self->priv->stream = (fclose (self->priv->stream), NULL)), _tmp0);
 	if (self->priv->file_exists) {
 		gboolean changed;
 		changed = TRUE;
@@ -112,29 +112,28 @@ void vala_ccode_writer_close (ValaCCodeWriter* self) {
 				if (inner_error->domain == G_FILE_ERROR) {
 					goto __catch0_g_file_error;
 				}
-				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
-				g_clear_error (&inner_error);
+				goto __finally0;
 			}
 			new_file = g_mapped_file_new (self->priv->temp_filename, FALSE, &inner_error);
 			if (inner_error != NULL) {
+				(old_file == NULL) ? NULL : (old_file = (g_mapped_file_free (old_file), NULL));
 				if (inner_error->domain == G_FILE_ERROR) {
 					goto __catch0_g_file_error;
 				}
-				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
-				g_clear_error (&inner_error);
+				goto __finally0;
 			}
 			len = g_mapped_file_get_length (old_file);
 			if (len == g_mapped_file_get_length (new_file)) {
-				if (memcmp (g_mapped_file_get_contents (old_file), g_mapped_file_get_contents (new_file), ((gsize) (len))) == 0) {
+				if (memcmp (g_mapped_file_get_contents (old_file), g_mapped_file_get_contents (new_file), (gsize) len) == 0) {
 					changed = FALSE;
 				}
 			}
 			_tmp1 = NULL;
-			old_file = (_tmp1 = NULL, (old_file == NULL ? NULL : (old_file = (g_mapped_file_free (old_file), NULL))), _tmp1);
+			old_file = (_tmp1 = NULL, (old_file == NULL) ? NULL : (old_file = (g_mapped_file_free (old_file), NULL)), _tmp1);
 			_tmp2 = NULL;
-			new_file = (_tmp2 = NULL, (new_file == NULL ? NULL : (new_file = (g_mapped_file_free (new_file), NULL))), _tmp2);
-			(old_file == NULL ? NULL : (old_file = (g_mapped_file_free (old_file), NULL)));
-			(new_file == NULL ? NULL : (new_file = (g_mapped_file_free (new_file), NULL)));
+			new_file = (_tmp2 = NULL, (new_file == NULL) ? NULL : (new_file = (g_mapped_file_free (new_file), NULL)), _tmp2);
+			(old_file == NULL) ? NULL : (old_file = (g_mapped_file_free (old_file), NULL));
+			(new_file == NULL) ? NULL : (new_file = (g_mapped_file_free (new_file), NULL));
 		}
 		goto __finally0;
 		__catch0_g_file_error:
@@ -143,11 +142,15 @@ void vala_ccode_writer_close (ValaCCodeWriter* self) {
 			e = inner_error;
 			inner_error = NULL;
 			{
-				(e == NULL ? NULL : (e = (g_error_free (e), NULL)));
+				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
 			}
 		}
 		__finally0:
-		;
+		if (inner_error != NULL) {
+			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
+			g_clear_error (&inner_error);
+			return;
+		}
 		/* assume changed if mmap comparison doesn't work*/
 		if (changed) {
 			g_rename (self->priv->temp_filename, self->priv->_filename);
@@ -162,9 +165,16 @@ void vala_ccode_writer_close (ValaCCodeWriter* self) {
  * Writes tabs according to the current indent level.
  */
 void vala_ccode_writer_write_indent (ValaCCodeWriter* self, ValaCCodeLineDirective* line) {
+	gboolean _tmp0;
 	g_return_if_fail (self != NULL);
-	if (self->priv->_line_directives && line != NULL) {
-		vala_ccode_node_write (((ValaCCodeNode*) (line)), self);
+	_tmp0 = FALSE;
+	if (self->priv->_line_directives) {
+		_tmp0 = line != NULL;
+	} else {
+		_tmp0 = FALSE;
+	}
+	if (_tmp0) {
+		vala_ccode_node_write ((ValaCCodeNode*) line, self);
 	}
 	if (!vala_ccode_writer_get_bol (self)) {
 		fputc ('\n', self->priv->stream);
@@ -238,8 +248,10 @@ void vala_ccode_writer_write_end_block (ValaCCodeWriter* self) {
  */
 void vala_ccode_writer_write_comment (ValaCCodeWriter* self, const char* text) {
 	gboolean first;
-	char** _tmp0;
+	char** _tmp1;
+	gint lines_size;
 	gint lines_length1;
+	char** _tmp0;
 	char** lines;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (text != NULL);
@@ -247,19 +259,20 @@ void vala_ccode_writer_write_comment (ValaCCodeWriter* self, const char* text) {
 	fprintf (self->priv->stream, "/*");
 	first = TRUE;
 	/* separate declaration due to missing memory management in foreach statements */
+	_tmp1 = NULL;
 	_tmp0 = NULL;
-	lines = (_tmp0 = g_strsplit (text, "\n", 0), lines_length1 = -1, _tmp0);
+	lines = (_tmp1 = _tmp0 = g_strsplit (text, "\n", 0), lines_length1 = g_strv_length (_tmp0), lines_size = lines_length1, _tmp1);
 	{
 		char** line_collection;
 		int line_collection_length1;
 		int line_it;
 		line_collection = lines;
 		line_collection_length1 = lines_length1;
-		for (line_it = 0; (lines_length1 != -1 && line_it < lines_length1) || (lines_length1 == -1 && line_collection[line_it] != NULL); line_it = line_it + 1) {
-			const char* _tmp1;
+		for (line_it = 0; line_it < lines_length1; line_it = line_it + 1) {
+			const char* _tmp2;
 			char* line;
-			_tmp1 = NULL;
-			line = (_tmp1 = line_collection[line_it], (_tmp1 == NULL ? NULL : g_strdup (_tmp1)));
+			_tmp2 = NULL;
+			line = (_tmp2 = line_collection[line_it], (_tmp2 == NULL) ? NULL : g_strdup (_tmp2));
 			{
 				if (!first) {
 					vala_ccode_writer_write_indent (self, NULL);
@@ -273,7 +286,7 @@ void vala_ccode_writer_write_comment (ValaCCodeWriter* self, const char* text) {
 	}
 	fprintf (self->priv->stream, "*/");
 	vala_ccode_writer_write_newline (self);
-	lines = (_vala_array_free (lines, lines_length1, ((GDestroyNotify) (g_free))), NULL);
+	lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL);
 }
 
 
@@ -289,7 +302,7 @@ void vala_ccode_writer_set_filename (ValaCCodeWriter* self, const char* value) {
 	g_return_if_fail (self != NULL);
 	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_filename = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (self->priv->_filename = (g_free (self->priv->_filename), NULL)), _tmp2);
+	self->priv->_filename = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), self->priv->_filename = (g_free (self->priv->_filename), NULL), _tmp2);
 }
 
 
@@ -340,12 +353,13 @@ static gpointer vala_value_ccode_writer_peek_pointer (const GValue* value) {
 static gchar* vala_value_ccode_writer_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
 	if (collect_values[0].v_pointer) {
 		ValaCCodeWriter* object;
-		object = value->data[0].v_pointer;
+		object = collect_values[0].v_pointer;
 		if (object->parent_instance.g_class == NULL) {
 			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
-		} else if (!g_value_type_compatible (G_OBJECT_TYPE (object), G_VALUE_TYPE (value))) {
-			return g_strconcat ("invalid object type `", G_OBJECT_TYPE (object), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
+		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
+			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
 		}
+		value->data[0].v_pointer = vala_ccode_writer_ref (object);
 	} else {
 		value->data[0].v_pointer = NULL;
 	}
@@ -357,6 +371,7 @@ static gchar* vala_value_ccode_writer_lcopy_value (const GValue* value, guint n_
 	ValaCCodeWriter** object_p;
 	object_p = collect_values[0].v_pointer;
 	if (!object_p) {
+		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
 	}
 	if (!value->data[0].v_pointer) {
 		*object_p = NULL;
@@ -421,7 +436,7 @@ static void vala_ccode_writer_finalize (ValaCCodeWriter* obj) {
 	self = VALA_CCODE_WRITER (obj);
 	self->priv->_filename = (g_free (self->priv->_filename), NULL);
 	self->priv->temp_filename = (g_free (self->priv->temp_filename), NULL);
-	(self->priv->stream == NULL ? NULL : (self->priv->stream = (fclose (self->priv->stream), NULL)));
+	(self->priv->stream == NULL) ? NULL : (self->priv->stream = (fclose (self->priv->stream), NULL));
 }
 
 
@@ -450,22 +465,18 @@ void vala_ccode_writer_unref (gpointer instance) {
 	self = instance;
 	if (g_atomic_int_dec_and_test (&self->ref_count)) {
 		VALA_CCODE_WRITER_GET_CLASS (self)->finalize (self);
-		g_type_free_instance (((GTypeInstance *) (self)));
+		g_type_free_instance ((GTypeInstance *) self);
 	}
 }
 
 
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
-	if (array != NULL && destroy_func != NULL) {
+	if ((array != NULL) && (destroy_func != NULL)) {
 		int i;
-		if (array_length >= 0)
 		for (i = 0; i < array_length; i = i + 1) {
-			if (((gpointer*) (array))[i] != NULL)
-			destroy_func (((gpointer*) (array))[i]);
-		}
-		else
-		for (i = 0; ((gpointer*) (array))[i] != NULL; i = i + 1) {
-			destroy_func (((gpointer*) (array))[i]);
+			if (((gpointer*) array)[i] != NULL) {
+				destroy_func (((gpointer*) array)[i]);
+			}
 		}
 	}
 	g_free (array);
