@@ -114,7 +114,6 @@ static void gee_map_base_init (GeeMapIface * iface) {
 	static gboolean initialized = FALSE;
 	if (!initialized) {
 		initialized = TRUE;
-		g_object_interface_install_property (iface, g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
 	}
 }
 
@@ -122,9 +121,9 @@ static void gee_map_base_init (GeeMapIface * iface) {
 GType gee_map_get_type (void) {
 	static GType gee_map_type_id = 0;
 	if (gee_map_type_id == 0) {
-		static const GTypeInfo g_define_type_info = { sizeof (GeeMapIface), (GBaseInitFunc) gee_map_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL };
+		static const GTypeInfo g_define_type_info = { sizeof (GeeMapIface), (GBaseInitFunc) gee_map_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
 		gee_map_type_id = g_type_register_static (G_TYPE_INTERFACE, "GeeMap", &g_define_type_info, 0);
-		g_type_interface_add_prerequisite (gee_map_type_id, G_TYPE_OBJECT);
+		g_type_interface_add_prerequisite (gee_map_type_id, GEE_TYPE_COLLECTION_OBJECT);
 	}
 	return gee_map_type_id;
 }
