@@ -1,6 +1,6 @@
 /* valaassignment.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter
+ * Copyright (C) 2006-2008  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,9 +25,9 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <gee/collection.h>
 #include <vala/valaexpression.h>
 #include <vala/valacodenode.h>
-#include <vala/valacodebinding.h>
 
 G_BEGIN_DECLS
 
