@@ -21,16 +21,17 @@
  */
 
 #include <vala/valacodenode.h>
-#include <gee/collection.h>
 #include <gee/readonlylist.h>
 #include <gee/arraylist.h>
+#include <gee/iterable.h>
+#include <gee/iterator.h>
 #include <ccode/valaccodelinedirective.h>
 #include <vala/valasourcereference.h>
 #include <vala/valacodevisitor.h>
-#include <vala/valadatatype.h>
 #include <vala/valasemanticanalyzer.h>
-#include <vala/valaexpression.h>
 #include <vala/valaattribute.h>
+#include <vala/valadatatype.h>
+#include <vala/valaexpression.h>
 #include <vala/valasourcefile.h>
 #include <gobject/gvaluecollector.h>
 
@@ -51,13 +52,16 @@ enum  {
 	VALA_CODE_NODE_DUMMY_PROPERTY
 };
 static void _g_list_free_vala_code_node_unref (GList* self);
-static GeeList* vala_code_node__empty_type_list;
+static GeeList* vala_code_node__empty_type_list = NULL;
+static gint vala_code_node_last_temp_nr = 0;
 static void vala_code_node_real_accept (ValaCodeNode* self, ValaCodeVisitor* visitor);
 static void vala_code_node_real_accept_children (ValaCodeNode* self, ValaCodeVisitor* visitor);
 static gboolean vala_code_node_real_check (ValaCodeNode* self, ValaSemanticAnalyzer* analyzer);
 static void vala_code_node_real_replace_type (ValaCodeNode* self, ValaDataType* old_type, ValaDataType* new_type);
 static void vala_code_node_real_replace_expression (ValaCodeNode* self, ValaExpression* old_node, ValaExpression* new_node);
 static char* vala_code_node_real_to_string (ValaCodeNode* self);
+static void vala_code_node_real_get_defined_variables (ValaCodeNode* self, GeeCollection* collection);
+static void vala_code_node_real_get_used_variables (ValaCodeNode* self, GeeCollection* collection);
 static gpointer vala_code_node_parent_class = NULL;
 static void vala_code_node_finalize (ValaCodeNode* obj);
 static int _vala_strcmp0 (const char * str1, const char * str2);
@@ -65,7 +69,7 @@ static int _vala_strcmp0 (const char * str1, const char * str2);
 
 
 static void _g_list_free_vala_code_node_unref (GList* self) {
-	g_list_foreach (self, ((GFunc) (vala_code_node_unref)), NULL);
+	g_list_foreach (self, (GFunc) vala_code_node_unref, NULL);
 	g_list_free (self);
 }
 
@@ -79,18 +83,18 @@ GeeList* vala_code_node_get_error_types (ValaCodeNode* self) {
 	if (self->priv->_error_types != NULL) {
 		GeeList* _tmp0;
 		_tmp0 = NULL;
-		return (_tmp0 = self->priv->_error_types, (_tmp0 == NULL ? NULL : gee_collection_object_ref (_tmp0)));
+		return (_tmp0 = self->priv->_error_types, (_tmp0 == NULL) ? NULL : gee_collection_object_ref (_tmp0));
 	}
 	if (vala_code_node__empty_type_list == NULL) {
 		GeeList* _tmp3;
 		GeeArrayList* _tmp2;
 		_tmp3 = NULL;
 		_tmp2 = NULL;
-		vala_code_node__empty_type_list = (_tmp3 = ((GeeList*) (gee_read_only_list_new (VALA_TYPE_DATA_TYPE, ((GBoxedCopyFunc) (vala_code_node_ref)), vala_code_node_unref, ((GeeList*) ((_tmp2 = gee_array_list_new (VALA_TYPE_DATA_TYPE, ((GBoxedCopyFunc) (vala_code_node_ref)), vala_code_node_unref, g_direct_equal))))))), (vala_code_node__empty_type_list == NULL ? NULL : (vala_code_node__empty_type_list = (gee_collection_object_unref (vala_code_node__empty_type_list), NULL))), _tmp3);
-		(_tmp2 == NULL ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL)));
+		vala_code_node__empty_type_list = (_tmp3 = (GeeList*) gee_read_only_list_new (VALA_TYPE_DATA_TYPE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, (GeeList*) (_tmp2 = gee_array_list_new (VALA_TYPE_DATA_TYPE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal))), (vala_code_node__empty_type_list == NULL) ? NULL : (vala_code_node__empty_type_list = (gee_collection_object_unref (vala_code_node__empty_type_list), NULL)), _tmp3);
+		(_tmp2 == NULL) ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL));
 	}
 	_tmp4 = NULL;
-	return (_tmp4 = vala_code_node__empty_type_list, (_tmp4 == NULL ? NULL : gee_collection_object_ref (_tmp4)));
+	return (_tmp4 = vala_code_node__empty_type_list, (_tmp4 == NULL) ? NULL : gee_collection_object_ref (_tmp4));
 }
 
 
@@ -104,10 +108,10 @@ void vala_code_node_add_error_type (ValaCodeNode* self, ValaDataType* error_type
 	if (self->priv->_error_types == NULL) {
 		GeeList* _tmp0;
 		_tmp0 = NULL;
-		self->priv->_error_types = (_tmp0 = ((GeeList*) (gee_array_list_new (VALA_TYPE_DATA_TYPE, ((GBoxedCopyFunc) (vala_code_node_ref)), vala_code_node_unref, g_direct_equal))), (self->priv->_error_types == NULL ? NULL : (self->priv->_error_types = (gee_collection_object_unref (self->priv->_error_types), NULL))), _tmp0);
+		self->priv->_error_types = (_tmp0 = (GeeList*) gee_array_list_new (VALA_TYPE_DATA_TYPE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal), (self->priv->_error_types == NULL) ? NULL : (self->priv->_error_types = (gee_collection_object_unref (self->priv->_error_types), NULL)), _tmp0);
 	}
-	gee_collection_add (((GeeCollection*) (self->priv->_error_types)), error_type);
-	vala_code_node_set_parent_node (((ValaCodeNode*) (error_type)), self);
+	gee_collection_add ((GeeCollection*) self->priv->_error_types, error_type);
+	vala_code_node_set_parent_node ((ValaCodeNode*) error_type, self);
 }
 
 
@@ -119,17 +123,15 @@ void vala_code_node_add_error_types (ValaCodeNode* self, GeeList* error_types) {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (error_types != NULL);
 	{
-		GeeList* error_type_collection;
-		int error_type_it;
-		error_type_collection = error_types;
-		for (error_type_it = 0; error_type_it < gee_collection_get_size (GEE_COLLECTION (error_type_collection)); error_type_it = error_type_it + 1) {
+		GeeIterator* _error_type_it;
+		_error_type_it = gee_iterable_iterator ((GeeIterable*) error_types);
+		while (gee_iterator_next (_error_type_it)) {
 			ValaDataType* error_type;
-			error_type = ((ValaDataType*) (gee_list_get (GEE_LIST (error_type_collection), error_type_it)));
-			{
-				vala_code_node_add_error_type (self, error_type);
-				(error_type == NULL ? NULL : (error_type = (vala_code_node_unref (error_type), NULL)));
-			}
+			error_type = (ValaDataType*) gee_iterator_get (_error_type_it);
+			vala_code_node_add_error_type (self, error_type);
+			(error_type == NULL) ? NULL : (error_type = (vala_code_node_unref (error_type), NULL));
 		}
+		(_error_type_it == NULL) ? NULL : (_error_type_it = (gee_collection_object_unref (_error_type_it), NULL));
 	}
 }
 
@@ -220,12 +222,12 @@ ValaAttribute* vala_code_node_get_attribute (ValaCodeNode* self, const char* nam
 			ValaAttribute* _tmp1;
 			ValaAttribute* a;
 			_tmp1 = NULL;
-			a = (_tmp1 = ((ValaAttribute*) (a_it->data)), (_tmp1 == NULL ? NULL : vala_code_node_ref (_tmp1)));
+			a = (_tmp1 = (ValaAttribute*) a_it->data, (_tmp1 == NULL) ? NULL : vala_code_node_ref (_tmp1));
 			{
 				if (_vala_strcmp0 (vala_attribute_get_name (a), name) == 0) {
 					return a;
 				}
-				(a == NULL ? NULL : (a = (vala_code_node_unref (a), NULL)));
+				(a == NULL) ? NULL : (a = (vala_code_node_unref (a), NULL));
 			}
 		}
 	}
@@ -248,12 +250,12 @@ static char* vala_code_node_real_to_string (ValaCodeNode* self) {
 	if (self->priv->_source_reference != NULL) {
 		char* _tmp0;
 		_tmp0 = NULL;
-		g_string_append (g_string_append (str, "@"), (_tmp0 = vala_source_reference_to_string (self->priv->_source_reference)));
+		g_string_append (g_string_append (str, "@"), _tmp0 = vala_source_reference_to_string (self->priv->_source_reference));
 		_tmp0 = (g_free (_tmp0), NULL);
 	}
 	_tmp1 = NULL;
 	_tmp2 = NULL;
-	return (_tmp2 = (_tmp1 = g_string_append (str, " */")->str, (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (str == NULL ? NULL : (str = (g_string_free (str, TRUE), NULL))), _tmp2);
+	return (_tmp2 = (_tmp1 = g_string_append (str, " */")->str, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), (str == NULL) ? NULL : (str = (g_string_free (str, TRUE), NULL)), _tmp2);
 }
 
 
@@ -262,6 +264,38 @@ char* vala_code_node_to_string (ValaCodeNode* self) {
 }
 
 
+static void vala_code_node_real_get_defined_variables (ValaCodeNode* self, GeeCollection* collection) {
+	g_return_if_fail (self != NULL);
+	g_return_if_fail (collection != NULL);
+}
+
+
+void vala_code_node_get_defined_variables (ValaCodeNode* self, GeeCollection* collection) {
+	VALA_CODE_NODE_GET_CLASS (self)->get_defined_variables (self, collection);
+}
+
+
+static void vala_code_node_real_get_used_variables (ValaCodeNode* self, GeeCollection* collection) {
+	g_return_if_fail (self != NULL);
+	g_return_if_fail (collection != NULL);
+}
+
+
+void vala_code_node_get_used_variables (ValaCodeNode* self, GeeCollection* collection) {
+	VALA_CODE_NODE_GET_CLASS (self)->get_used_variables (self, collection);
+}
+
+
+char* vala_code_node_get_temp_name (ValaCodeNode* self) {
+	char* _tmp0;
+	char* _tmp1;
+	g_return_val_if_fail (self != NULL, NULL);
+	_tmp0 = NULL;
+	_tmp1 = NULL;
+	return (_tmp1 = g_strconcat (".", _tmp0 = g_strdup_printf ("%i", ((vala_code_node_last_temp_nr = vala_code_node_last_temp_nr + 1))), NULL), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
+}
+
+
 ValaCodeNode* vala_code_node_get_parent_node (ValaCodeNode* self) {
 	g_return_val_if_fail (self != NULL, NULL);
 	return self->priv->_parent_node;
@@ -286,7 +320,7 @@ void vala_code_node_set_source_reference (ValaCodeNode* self, ValaSourceReferenc
 	g_return_if_fail (self != NULL);
 	_tmp2 = NULL;
 	_tmp1 = NULL;
-	self->priv->_source_reference = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : vala_source_reference_ref (_tmp1))), (self->priv->_source_reference == NULL ? NULL : (self->priv->_source_reference = (vala_source_reference_unref (self->priv->_source_reference), NULL))), _tmp2);
+	self->priv->_source_reference = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : vala_source_reference_ref (_tmp1)), (self->priv->_source_reference == NULL) ? NULL : (self->priv->_source_reference = (vala_source_reference_unref (self->priv->_source_reference), NULL)), _tmp2);
 }
 
 
@@ -297,21 +331,28 @@ ValaCCodeNode* vala_code_node_get_ccodenode (ValaCodeNode* self) {
 
 
 void vala_code_node_set_ccodenode (ValaCodeNode* self, ValaCCodeNode* value) {
+	gboolean _tmp1;
+	ValaCCodeNode* _tmp5;
 	ValaCCodeNode* _tmp4;
-	ValaCCodeNode* _tmp3;
 	g_return_if_fail (self != NULL);
-	if (value != NULL && self->priv->_source_reference != NULL) {
-		ValaCCodeLineDirective* _tmp2;
-		char* _tmp1;
-		_tmp2 = NULL;
-		_tmp1 = NULL;
-		vala_ccode_node_set_line (value, (_tmp2 = vala_ccode_line_directive_new ((_tmp1 = g_path_get_basename (vala_source_file_get_filename (vala_source_reference_get_file (self->priv->_source_reference)))), vala_source_reference_get_first_line (self->priv->_source_reference))));
-		(_tmp2 == NULL ? NULL : (_tmp2 = (vala_ccode_node_unref (_tmp2), NULL)));
-		_tmp1 = (g_free (_tmp1), NULL);
+	_tmp1 = FALSE;
+	if (value != NULL) {
+		_tmp1 = self->priv->_source_reference != NULL;
+	} else {
+		_tmp1 = FALSE;
 	}
+	if (_tmp1) {
+		ValaCCodeLineDirective* _tmp3;
+		char* _tmp2;
+		_tmp3 = NULL;
+		_tmp2 = NULL;
+		vala_ccode_node_set_line (value, _tmp3 = vala_ccode_line_directive_new (_tmp2 = g_path_get_basename (vala_source_file_get_filename (vala_source_reference_get_file (self->priv->_source_reference))), vala_source_reference_get_first_line (self->priv->_source_reference)));
+		(_tmp3 == NULL) ? NULL : (_tmp3 = (vala_ccode_node_unref (_tmp3), NULL));
+		_tmp2 = (g_free (_tmp2), NULL);
+	}
+	_tmp5 = NULL;
 	_tmp4 = NULL;
-	_tmp3 = NULL;
-	self->priv->_ccodenode = (_tmp4 = (_tmp3 = value, (_tmp3 == NULL ? NULL : vala_ccode_node_ref (_tmp3))), (self->priv->_ccodenode == NULL ? NULL : (self->priv->_ccodenode = (vala_ccode_node_unref (self->priv->_ccodenode), NULL))), _tmp4);
+	self->priv->_ccodenode = (_tmp5 = (_tmp4 = value, (_tmp4 == NULL) ? NULL : vala_ccode_node_ref (_tmp4)), (self->priv->_ccodenode == NULL) ? NULL : (self->priv->_ccodenode = (vala_ccode_node_unref (self->priv->_ccodenode), NULL)), _tmp5);
 }
 
 
@@ -340,8 +381,15 @@ void vala_code_node_set_error (ValaCodeNode* self, gboolean value) {
 
 
 gboolean vala_code_node_get_tree_can_fail (ValaCodeNode* self) {
+	gboolean _tmp0;
 	g_return_val_if_fail (self != NULL, FALSE);
-	return self->priv->_error_types != NULL && gee_collection_get_size (((GeeCollection*) (self->priv->_error_types))) > 0;
+	_tmp0 = FALSE;
+	if (self->priv->_error_types != NULL) {
+		_tmp0 = gee_collection_get_size ((GeeCollection*) self->priv->_error_types) > 0;
+	} else {
+		_tmp0 = FALSE;
+	}
+	return _tmp0;
 }
 
 
@@ -374,12 +422,13 @@ static gpointer vala_value_code_node_peek_pointer (const GValue* value) {
 static gchar* vala_value_code_node_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
 	if (collect_values[0].v_pointer) {
 		ValaCodeNode* object;
-		object = value->data[0].v_pointer;
+		object = collect_values[0].v_pointer;
 		if (object->parent_instance.g_class == NULL) {
 			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
-		} else if (!g_value_type_compatible (G_OBJECT_TYPE (object), G_VALUE_TYPE (value))) {
-			return g_strconcat ("invalid object type `", G_OBJECT_TYPE (object), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
+		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
+			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
 		}
+		value->data[0].v_pointer = vala_code_node_ref (object);
 	} else {
 		value->data[0].v_pointer = NULL;
 	}
@@ -391,6 +440,7 @@ static gchar* vala_value_code_node_lcopy_value (const GValue* value, guint n_col
 	ValaCodeNode** object_p;
 	object_p = collect_values[0].v_pointer;
 	if (!object_p) {
+		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
 	}
 	if (!value->data[0].v_pointer) {
 		*object_p = NULL;
@@ -446,6 +496,8 @@ static void vala_code_node_class_init (ValaCodeNodeClass * klass) {
 	VALA_CODE_NODE_CLASS (klass)->replace_type = vala_code_node_real_replace_type;
 	VALA_CODE_NODE_CLASS (klass)->replace_expression = vala_code_node_real_replace_expression;
 	VALA_CODE_NODE_CLASS (klass)->to_string = vala_code_node_real_to_string;
+	VALA_CODE_NODE_CLASS (klass)->get_defined_variables = vala_code_node_real_get_defined_variables;
+	VALA_CODE_NODE_CLASS (klass)->get_used_variables = vala_code_node_real_get_used_variables;
 }
 
 
@@ -458,10 +510,10 @@ static void vala_code_node_instance_init (ValaCodeNode * self) {
 static void vala_code_node_finalize (ValaCodeNode* obj) {
 	ValaCodeNode * self;
 	self = VALA_CODE_NODE (obj);
-	(self->priv->_source_reference == NULL ? NULL : (self->priv->_source_reference = (vala_source_reference_unref (self->priv->_source_reference), NULL)));
-	(self->attributes == NULL ? NULL : (self->attributes = (_g_list_free_vala_code_node_unref (self->attributes), NULL)));
-	(self->priv->_error_types == NULL ? NULL : (self->priv->_error_types = (gee_collection_object_unref (self->priv->_error_types), NULL)));
-	(self->priv->_ccodenode == NULL ? NULL : (self->priv->_ccodenode = (vala_ccode_node_unref (self->priv->_ccodenode), NULL)));
+	(self->priv->_source_reference == NULL) ? NULL : (self->priv->_source_reference = (vala_source_reference_unref (self->priv->_source_reference), NULL));
+	(self->attributes == NULL) ? NULL : (self->attributes = (_g_list_free_vala_code_node_unref (self->attributes), NULL));
+	(self->priv->_error_types == NULL) ? NULL : (self->priv->_error_types = (gee_collection_object_unref (self->priv->_error_types), NULL));
+	(self->priv->_ccodenode == NULL) ? NULL : (self->priv->_ccodenode = (vala_ccode_node_unref (self->priv->_ccodenode), NULL));
 }
 
 
@@ -490,7 +542,7 @@ void vala_code_node_unref (gpointer instance) {
 	self = instance;
 	if (g_atomic_int_dec_and_test (&self->ref_count)) {
 		VALA_CODE_NODE_GET_CLASS (self)->finalize (self);
-		g_type_free_instance (((GTypeInstance *) (self)));
+		g_type_free_instance ((GTypeInstance *) self);
 	}
 }
 
@@ -500,7 +552,7 @@ static int _vala_strcmp0 (const char * str1, const char * str2) {
 		return -(str1 != str2);
 	}
 	if (str2 == NULL) {
-		return (str1 != str2);
+		return str1 != str2;
 	}
 	return strcmp (str1, str2);
 }
