@@ -1,6 +1,6 @@
 /* valafield.vala
  *
- * Copyright (C) 2006-2007  Jürg Billeter
+ * Copyright (C) 2006-2009  Jürg Billeter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -50,24 +50,35 @@ struct _ValaField {
 	ValaMember parent_instance;
 	ValaFieldPrivate * priv;
 };
+
 struct _ValaFieldClass {
 	ValaMemberClass parent_class;
 };
 
-ValaField* vala_field_new (const char* name, ValaDataType* type_reference, ValaExpression* initializer, ValaSourceReference* source_reference);
+
+ValaField* vala_field_construct (GType object_type, const char* name, ValaDataType* field_type, ValaExpression* initializer, ValaSourceReference* source_reference);
+ValaField* vala_field_new (const char* name, ValaDataType* field_type, ValaExpression* initializer, ValaSourceReference* source_reference);
 char* vala_field_get_cname (ValaField* self);
+void vala_field_set_cname (ValaField* self, const char* cname);
 char* vala_field_get_default_cname (ValaField* self);
+char* vala_field_get_array_length_cname (ValaField* self);
+void vala_field_set_array_length_cname (ValaField* self, const char* array_length_cname);
 void vala_field_process_attributes (ValaField* self);
-ValaDataType* vala_field_get_type_reference (ValaField* self);
-void vala_field_set_type_reference (ValaField* self, ValaDataType* value);
+char* vala_field_get_ctype (ValaField* self);
+void vala_field_set_ctype (ValaField* self, const char* ctype);
+ValaDataType* vala_field_get_field_type (ValaField* self);
+void vala_field_set_field_type (ValaField* self, ValaDataType* value);
 ValaExpression* vala_field_get_initializer (ValaField* self);
 void vala_field_set_initializer (ValaField* self, ValaExpression* value);
-gboolean vala_field_get_instance (ValaField* self);
-void vala_field_set_instance (ValaField* self, gboolean value);
+MemberBinding vala_field_get_binding (ValaField* self);
+void vala_field_set_binding (ValaField* self, MemberBinding value);
 gboolean vala_field_get_is_volatile (ValaField* self);
 void vala_field_set_is_volatile (ValaField* self, gboolean value);
 gboolean vala_field_get_no_array_length (ValaField* self);
 void vala_field_set_no_array_length (ValaField* self, gboolean value);
+gboolean vala_field_get_array_null_terminated (ValaField* self);
+void vala_field_set_array_null_terminated (ValaField* self, gboolean value);
+gboolean vala_field_get_has_array_length_cname (ValaField* self);
 GType vala_field_get_type (void);
 
 
