@@ -33,9 +33,9 @@
 #include <vala/valasemanticanalyzer.h>
 #include <vala/valadelegatetype.h>
 #include <vala/valareport.h>
-#include <vala/valamember.h>
 #include <vala/valadelegate.h>
 #include <vala/valadatatype.h>
+#include <vala/valamember.h>
 #include <vala/valasymbol.h>
 #include <vala/valaformalparameter.h>
 #include <vala/valascope.h>
@@ -184,8 +184,6 @@ static char* vala_lambda_expression_get_lambda_name (ValaLambdaExpression* self,
 
 static gboolean vala_lambda_expression_real_check (ValaCodeNode* base, ValaSemanticAnalyzer* analyzer) {
 	ValaLambdaExpression * self;
-	gboolean in_instance_method;
-	ValaMethod* current_method;
 	ValaDelegate* _tmp2;
 	ValaDelegate* cb;
 	ValaMethod* _tmp4;
@@ -206,13 +204,6 @@ static gboolean vala_lambda_expression_real_check (ValaCodeNode* base, ValaSeman
 		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), "lambda expression not allowed in this context");
 		return FALSE;
 	}
-	in_instance_method = FALSE;
-	current_method = vala_semantic_analyzer_find_current_method (analyzer);
-	if (current_method != NULL) {
-		in_instance_method = (vala_method_get_binding (current_method) == MEMBER_BINDING_INSTANCE);
-	} else {
-		in_instance_method = vala_semantic_analyzer_is_in_constructor (analyzer);
-	}
 	_tmp2 = NULL;
 	cb = (_tmp2 = VALA_DELEGATE (vala_delegate_type_get_delegate_symbol ((VALA_DELEGATE_TYPE (vala_expression_get_target_type ((ValaExpression*) self))))), (_tmp2 == NULL) ? NULL : vala_code_node_ref (_tmp2));
 	_tmp4 = NULL;
@@ -224,7 +215,7 @@ static gboolean vala_lambda_expression_real_check (ValaCodeNode* base, ValaSeman
 	if (!vala_delegate_get_has_target (cb)) {
 		_tmp5 = TRUE;
 	} else {
-		_tmp5 = !in_instance_method;
+		_tmp5 = !vala_semantic_analyzer_is_in_instance_method (analyzer);
 	}
 	if (_tmp5) {
 		vala_method_set_binding (self->priv->_method, MEMBER_BINDING_STATIC);
@@ -235,15 +226,15 @@ static gboolean vala_lambda_expression_real_check (ValaCodeNode* base, ValaSeman
 	{
 		GeeList* _tmp6;
 		GeeIterator* _tmp7;
-		GeeIterator* cb_param_it;
+		GeeIterator* _cb_param_it;
 		_tmp6 = NULL;
 		_tmp7 = NULL;
-		cb_param_it = (_tmp7 = gee_iterable_iterator ((GeeIterable*) (_tmp6 = vala_delegate_get_parameters (cb))), (_tmp6 == NULL) ? NULL : (_tmp6 = (gee_collection_object_unref (_tmp6), NULL)), _tmp7);
-		while (gee_iterator_next (cb_param_it)) {
+		_cb_param_it = (_tmp7 = gee_iterable_iterator ((GeeIterable*) (_tmp6 = vala_delegate_get_parameters (cb))), (_tmp6 == NULL) ? NULL : (_tmp6 = (gee_collection_object_unref (_tmp6), NULL)), _tmp7);
+		while (gee_iterator_next (_cb_param_it)) {
 			ValaFormalParameter* cb_param;
 			char* lambda_param;
 			ValaFormalParameter* param;
-			cb_param = (ValaFormalParameter*) gee_iterator_get (cb_param_it);
+			cb_param = (ValaFormalParameter*) gee_iterator_get (_cb_param_it);
 			if (!gee_iterator_next (lambda_param_it)) {
 				/* lambda expressions are allowed to have less parameters */
 				(cb_param == NULL) ? NULL : (cb_param = (vala_code_node_unref (cb_param), NULL));
@@ -256,14 +247,14 @@ static gboolean vala_lambda_expression_real_check (ValaCodeNode* base, ValaSeman
 			lambda_param = (g_free (lambda_param), NULL);
 			(param == NULL) ? NULL : (param = (vala_code_node_unref (param), NULL));
 		}
-		(cb_param_it == NULL) ? NULL : (cb_param_it = (gee_collection_object_unref (cb_param_it), NULL));
+		(_cb_param_it == NULL) ? NULL : (_cb_param_it = (gee_collection_object_unref (_cb_param_it), NULL));
 	}
 	if (gee_iterator_next (lambda_param_it)) {
 		gboolean _tmp8;
 		/* lambda expressions may not expect more parameters */
 		vala_code_node_set_error ((ValaCodeNode*) self, TRUE);
 		vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) self), "lambda expression: too many parameters");
-		return (_tmp8 = FALSE, (current_method == NULL) ? NULL : (current_method = (vala_code_node_unref (current_method), NULL)), (cb == NULL) ? NULL : (cb = (vala_code_node_unref (cb), NULL)), (lambda_params == NULL) ? NULL : (lambda_params = (gee_collection_object_unref (lambda_params), NULL)), (lambda_param_it == NULL) ? NULL : (lambda_param_it = (gee_collection_object_unref (lambda_param_it), NULL)), _tmp8);
+		return (_tmp8 = FALSE, (cb == NULL) ? NULL : (cb = (vala_code_node_unref (cb), NULL)), (lambda_params == NULL) ? NULL : (lambda_params = (gee_collection_object_unref (lambda_params), NULL)), (lambda_param_it == NULL) ? NULL : (lambda_param_it = (gee_collection_object_unref (lambda_param_it), NULL)), _tmp8);
 	}
 	if (self->priv->_expression_body != NULL) {
 		ValaBlock* block;
@@ -302,7 +293,7 @@ static gboolean vala_lambda_expression_real_check (ValaCodeNode* base, ValaSeman
 	_tmp11 = NULL;
 	vala_expression_set_value_type ((ValaExpression*) self, (ValaDataType*) (_tmp11 = vala_method_type_new (self->priv->_method)));
 	(_tmp11 == NULL) ? NULL : (_tmp11 = (vala_code_node_unref (_tmp11), NULL));
-	return (_tmp12 = !vala_code_node_get_error ((ValaCodeNode*) self), (current_method == NULL) ? NULL : (current_method = (vala_code_node_unref (current_method), NULL)), (cb == NULL) ? NULL : (cb = (vala_code_node_unref (cb), NULL)), (lambda_params == NULL) ? NULL : (lambda_params = (gee_collection_object_unref (lambda_params), NULL)), (lambda_param_it == NULL) ? NULL : (lambda_param_it = (gee_collection_object_unref (lambda_param_it), NULL)), _tmp12);
+	return (_tmp12 = !vala_code_node_get_error ((ValaCodeNode*) self), (cb == NULL) ? NULL : (cb = (vala_code_node_unref (cb), NULL)), (lambda_params == NULL) ? NULL : (lambda_params = (gee_collection_object_unref (lambda_params), NULL)), (lambda_param_it == NULL) ? NULL : (lambda_param_it = (gee_collection_object_unref (lambda_param_it), NULL)), _tmp12);
 }
 
 
