@@ -62,7 +62,7 @@ gint vala_report_get_errors (void) {
  */
 void vala_report_report_source (ValaSourceReference* source) {
 	char* offending_line;
-	g_return_if_fail (VALA_IS_SOURCE_REFERENCE (source));
+	g_return_if_fail (source != NULL);
 	if (vala_source_reference_get_first_line (source) != vala_source_reference_get_last_line (source)) {
 		/* FIXME Cannot report multi-line issues currently*/
 		return;
@@ -102,7 +102,6 @@ void vala_report_report_source (ValaSourceReference* source) {
  * @param message warning message
  */
 void vala_report_warning (ValaSourceReference* source, const char* message) {
-	g_return_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source));
 	g_return_if_fail (message != NULL);
 	vala_report_warnings++;
 	if (source == NULL) {
@@ -110,7 +109,7 @@ void vala_report_warning (ValaSourceReference* source, const char* message) {
 	} else {
 		char* _tmp0;
 		_tmp0 = NULL;
-		fprintf (stderr, "%s: warning: %s\n", (_tmp0 = vala_source_reference_to_string (source)), message);
+		fprintf (stderr, "%s: warning: %s\n", _tmp0 = vala_source_reference_to_string (source), message);
 		_tmp0 = (g_free (_tmp0), NULL);
 		if (vala_report_verbose_errors) {
 			vala_report_report_source (source);
@@ -126,7 +125,6 @@ void vala_report_warning (ValaSourceReference* source, const char* message) {
  * @param message error message
  */
 void vala_report_error (ValaSourceReference* source, const char* message) {
-	g_return_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source));
 	g_return_if_fail (message != NULL);
 	vala_report_errors++;
 	if (source == NULL) {
@@ -134,7 +132,7 @@ void vala_report_error (ValaSourceReference* source, const char* message) {
 	} else {
 		char* _tmp0;
 		_tmp0 = NULL;
-		fprintf (stderr, "%s: error: %s\n", (_tmp0 = vala_source_reference_to_string (source)), message);
+		fprintf (stderr, "%s: error: %s\n", _tmp0 = vala_source_reference_to_string (source), message);
 		_tmp0 = (g_free (_tmp0), NULL);
 		if (vala_report_verbose_errors) {
 			vala_report_report_source (source);
